#!/usr/bin/perl

use strict;
use warnings;
#use File::Copy;
#use IO::Handle;

use constant {
  STATE_ID => 1,
  STATE_METADATA => 2
};

# redirect output
#open OUTPUT, ">>", "/home/user/osaBackupLog";
#open ERROR,  ">>", "/home/user/osaBackupLog";
#STDOUT->fdopen( \*OUTPUT, 'w');
#STDERR->fdopen( \*ERROR,  'w');

sub trim {
  my $str = shift;
  $str =~ s/[\r\n]*$//;
  return $str
}

my $USER = "osa";
my ($login,$pass,$UID,$GID) = getpwnam($USER) or die("Unable to get uid/gid for '$USER'");
my $SOURCE = "/var/tmp/osaBackup";
my $TARGET = "/var/lib/ovi-adapter/known-products";

print "Source: $SOURCE\n";
print "Target: $TARGET\n";

sub merge {
  my $source = shift;
  my $target = shift;
  print "Merging '$source' -> '$target'\n";
  
  my %packageData;
  my $state = STATE_ID;
  my $id;
  
  open SOURCE, "<", $source or die $!;
  while (<SOURCE>) {
    my $line = trim($_);
    if ($state == STATE_ID) {
      $id = $line;
      print "Source id: $id\n";
      $state = STATE_METADATA;
      $packageData{$id} = []
    } else {
      if ($line eq "") { $state = STATE_ID }
      else { push(@{$packageData{$id}}, $line) }
    }
  }
  close SOURCE;
  
  $state = STATE_ID;
  
  open TARGET, "+<", $target or die $!;
  while (<TARGET>) {
    my $line = trim($_);
    if ($state == STATE_ID) {
      $id = $line;
      print "Target id: $id\n";
      $state = STATE_METADATA;
      if (exists $packageData{$id}) {
        print "Match\n";
        delete $packageData{$id}
      }
      
    } elsif ($line eq "") { $state = STATE_ID }
  }
  
  if (keys(%packageData) > 0) {
    print TARGET "\n";
    
    while (my ($id,$metadata) = each(%packageData)) {
      print "Writing $id\n";
      my $utc = time;
      print TARGET "$id\nOSA-Restored-UTC: $utc\n";
      foreach my $line (@{$metadata}) {
        print TARGET "$line\n"
      }
      print TARGET "\n"
    }
  }
  
  close TARGET;
}

# check if TARGET is set
if ($TARGET eq "") {
  print "Invalid target\n";
  exit 1
}

# check that SOURCE file exists
if (!-f $SOURCE) {
  print "Source not found\n";
  exit 2
}

# merge or copy, depending on target file existence
if (-f $TARGET) {
  merge($SOURCE,$TARGET)
}
else {
  print "Copying source to target\n";
  `cp "$SOURCE" "$TARGET")`;
  if (!-f $TARGET) {
    print "Unable to copy\n";
    exit 3
  }
}
    
# set owner and mode
my $cnt = chmod 0660, $TARGET;
if ($cnt != 1) {
  print "Failed to chmod target\n";
  exit 4
}

$cnt = chown $UID, $GID, $TARGET;
if ($cnt != 1) {
  print "Failed to chown target\n";
  exit 5
}

print "Restore ok\n";
exit 0
